/** PSP helper library ***************************************/
/**                                                         **/
/**                          image.c                        **/
/**                                                         **/
/** This file contains image manipulation routines          **/
/**                                                         **/
/** Copyright (C) Akop Karapetyan 2007                      **/
/**     You are not allowed to distribute this software     **/
/**     commercially. Please, notify me, if you make any    **/
/**     changes to this file.                               **/
/*************************************************************/
#include <malloc.h>
#include <string.h>

#include "image.h"

static void pspImageDrawHLine(PspImage *image, int sx, int dx, int y, unsigned short color);
static void pspImageDrawVLine(PspImage *image, int x, int sy, int dy, unsigned short color);

PspImage* pspImageCreate(int width, int height)
{
  int size = width * height * sizeof(unsigned short);
  unsigned short* pixels = (unsigned short*)malloc(size);

  if (!pixels) return NULL;

  PspImage *image =
    (PspImage*)malloc(sizeof(PspImage));

  if (!image)
  {
    free(pixels);
    return NULL;
  }

  memset(pixels, 0, size);

  image->Width = width;
  image->Height = height;
  image->Pixels = pixels;

  return image;
}

void pspImageDestroy(PspImage *image)
{
  free(image->Pixels);
  free(image);
}

PspImage* pspImageCreateThumbnail(const PspImage *image)
{
  PspImage *thumb;
  int i, j, p;

  if (!(thumb = pspImageCreate(image->Width / 2, image->Height / 2)))
    return NULL;

  for (i = 0, p = 0; i < image->Height; i += 2)
    for (j = 0; j < image->Width; j += 2)
      thumb->Pixels[p++] = image->Pixels[(image->Width * i) + j];

  return thumb;  
}

void pspImageDrawRect(PspImage *image, int sx, int sy, int dx, int dy, unsigned short color)
{
  pspImageDrawHLine(image, sx, dx, sy, color);
  pspImageDrawHLine(image, sx, dx, dy, color);
  pspImageDrawVLine(image, sx, sy, dy, color);
  pspImageDrawVLine(image, dx, sy, dy, color);
}

void pspImageFillRect(PspImage *image, int sx, int sy, int dx, int dy, unsigned short color)
{
  int minx, maxx, miny, maxy, i, j;
  unsigned short *pixel;

  /* Clamp coords */
  sx = (sx < 0) ? 0 : (sx >= image->Width) ? image->Width - 1 : sx;
  dx = (dx < 0) ? 0 : (dx >= image->Width) ? image->Width - 1 : dx;
  sy = (sy < 0) ? 0 : (sy >= image->Height) ? image->Height - 1 : sy;
  dy = (dy < 0) ? 0 : (dy >= image->Height) ? image->Height - 1 : dy;

  /* Determine smaller/larger */
  minx = (sx > dx) ? dx : sx;
  maxx = (sx < dx) ? dx : sx;
  miny = (sy > dy) ? dy : sy;
  maxy = (sy < dy) ? dy : sy;

  for (i = miny, pixel = image->Pixels + (miny * image->Width + minx); i <= maxy; i++, pixel += image->Width - (maxx - minx) - 1)
    for (j = minx; j <= maxx; j++, pixel++)
      *pixel = color;
}

void pspImagePrint(PspImage *image, const PspFont *font, char *message, int sx, int dx, unsigned short color)
{
}

static void pspImageDrawHLine(PspImage *image, int sx, int dx, int y, unsigned short color)
{
  int min, max, i;
  unsigned short *pixel;

  /* Clamp coords */
  sx = (sx < 0) ? 0 : (sx >= image->Width) ? image->Width - 1 : sx;
  dx = (dx < 0) ? 0 : (dx >= image->Width) ? image->Width - 1 : dx;
  y = (y < 0) ? 0 : (y >= image->Height) ? image->Height - 1 : y;

  /* Determine smaller/larger */
  min = (sx > dx) ? dx : sx;
  max = (sx < dx) ? dx : sx;

  for (i = min, pixel = image->Pixels + (y * image->Width + min); i <= max; i++, pixel++)
    *pixel = color;
}

static void pspImageDrawVLine(PspImage *image, int x, int sy, int dy, unsigned short color)
{
  int min, max, i;
  unsigned short *pixel;

  /* Clamp coords */
  sy = (sy < 0) ? 0 : (sy >= image->Height) ? image->Height - 1 : sy;
  dy = (dy < 0) ? 0 : (dy >= image->Height) ? image->Height - 1 : dy;
  x = (x < 0) ? 0 : (x >= image->Width) ? image->Width - 1 : x;

  /* Determine smaller/larger */
  min = (sy > dy) ? dy : sy;
  max = (sy < dy) ? dy : sy;

  for (i = min, pixel = image->Pixels + (min * image->Width + x); i <= max; i++, pixel += image->Width)
    *pixel = color;
}
